﻿using System;
using System.Collections.Generic;
using Ozeki.MediaGateway;


namespace Web2Web_Voice_Server
{
    /// <summary>
    /// Implements the server functionalities.
    /// </summary>
    class Web2WebGateway : MediaGateway
    {
        private Dictionary<IClient,MyClient> _clients;
        private int _clientCounter;
        private int _busyClients;

        public Web2WebGateway()
        {
            Console.WriteLine("Web2Web Gateway starting...");
        }

        #region Indications

        public override void OnStart()
        {
            _clients = new Dictionary<IClient, MyClient>();
            Console.WriteLine("Web2Web Gateway started.");
        }

        public override void OnClientConnect(IClient client, object[] parameters)
        {
            Console.WriteLine( "{0} client connected to the server.",client.RemoteAddress);
            if (!_clients.ContainsKey(client))
            {
                _clients.Add(client, new MyClient(client, string.Format("client{0}", _clientCounter++))); 
                NotifyClientsAboutTheirCallStatus();
            }
        }

        public override void OnClientDisconnect(IClient client)
        {
            if (_clients.ContainsKey(client))
            {
                MyClient disconnectedClient = _clients[client];
                if (disconnectedClient.IsBusy && disconnectedClient.RemoteParty!=null)
                    disconnectedClient.RemoteParty.OnCallStop();
                Console.WriteLine("{0}, {1} disconnected from the server.", client.RemoteAddress,disconnectedClient.Name);
                _clients.Remove(client);
                NotifyClientsAboutTheirCallStatus();
                return;
            }
            Console.WriteLine("{0} client disconnected from the server.", client.RemoteAddress);
        }

        public override void OnStreamPublishStart(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("client : {0} publish his stream : {1}",client.RemoteAddress,mediaStream.Name);
            base.OnStreamPublishStart(client, mediaStream);
        }

        #endregion

        #region Client Actions

        /// <summary>
        /// A connected client calls the first not busy client.
        /// </summary>
        /// <param name="invokerClient">The reference of the client who invoked this method.</param>
        /// <param name="requestOwner">The caller client's name.</param>
        public void Call(IClient invokerClient, string requestOwner)
        {
            foreach (KeyValuePair<IClient, MyClient> keyValuePair in _clients)
            {
                //Searchs the first not busy connected client and sets theirs remote party.
                if (keyValuePair.Key != invokerClient && !keyValuePair.Value.IsBusy)
                {
                    MyClient invoker = _clients[invokerClient];
                    MyClient callee = keyValuePair.Value;
                    invoker.RemoteParty = callee;
                    callee.RemoteParty = invoker;
                    callee.OnCallRequest(requestOwner);
                    return;
                }
            }
        }

        /// <summary>
        /// A hívott fél hívja ezt a metódust hogy a szerver elkezdje a hang közvetítésének szinkronizációs feladatát.
        /// </summary>
        /// <param name="client"></param>
        public void ChangeToIncall(IClient client)
        {
            _clients[client].OnInCall();
            foreach (MyClient c in _clients.Values)
            {
                NotifyClientsAboutTheirCallStatus();
            }
            
        }


        public void PlayRemoteStream(IClient client)
        {
            //foreach (MyClient c in Clients.Values)
            {
                _clients[client].OnPlayRemoteStream();
            }
        }

        public void CallStop(IClient invokerClient)
        {
            if (_clients.ContainsKey(invokerClient))
            {
                MyClient invoker = _clients[invokerClient];
                invoker.RemoteParty.OnCallStop();
            }
        }



        private void NotifyClientsAboutTheirCallStatus()
        {
            _busyClients = 0;
            foreach (MyClient c in _clients.Values)
            {
                if (c.IsBusy)
                    _busyClients++;
            }
            bool isReady = _clients.Count > 1 && _clients.Count-_busyClients > 1;

            lock (_clients)
            {
                foreach (KeyValuePair<IClient, MyClient> keyValuePair in _clients)
                {
                    if (!keyValuePair.Value.IsBusy)
                        keyValuePair.Value.OnSetReadyStatus(isReady, keyValuePair.Value.Name);
                }             
            }
        }

        #endregion
    }
}
